@@ -1,4 +1,4 @@
-// $Id: SessionPage.cpp,v 1.80 2006/05/28 10:05:01 manuelbi Exp $
+// $Id: SessionPage.cpp,v 1.84 2007/03/20 20:48:20 bifimsx Exp $
 // SessionPage.cpp: implementation of the SessionPage class.
 //
 //////////////////////////////////////////////////////////////////////
@@ -255,6 +255,7 @@ void SessionPage::EjectDisk (mediaInfo * target)
 	target->ips.Clear();
 	target->mmenu->SetLabel(Disk_Browse_Ips,wxT("Select IPS Patches (None selected)"));
 	target->lastContents = wxT("");
+	target->oldContents = wxT("");
 	wxString cmd = target->deviceName + wxT(" eject");
 	m_controller->WriteCommand(cmd);
 }
@@ -268,6 +269,7 @@ void SessionPage::EjectCart(mediaInfo * target)
 	target->control->SetSelection(wxNOT_FOUND);
 	target->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
 	target->mmenu->SetLabel(Cart_Select_Mapper,wxT("Select cartridge type (AUTO)"));
+	target->oldContents = wxT("");
 }
 
 void SessionPage::OnEjectCartByMenu (wxCommandEvent & event)
@@ -481,12 +483,16 @@ void SessionPage::OnBrowseCassette(wxCommandEvent &event)
 
 void SessionPage::OnClickDiskACombo(wxCommandEvent & event)
 {
+	wxComboBox * box = (wxComboBox *)event.GetEventObject();
+	wxString sel = box->GetString(box->GetSelection());
 	OnClickCombo(event);
-	OnChangeDiskAContents(event);
-	if (m_controller->IsOpenMSXRunning()) {
-		m_controller->WriteCommand(wxT("diska eject"));
-		if (!m_diskA->contents.IsEmpty()) {
-			m_controller->WriteCommand(wxT("diska ") + ConvertPath(m_diskA->contents,true));
+	if (sel != box->GetString(box->GetSelection())) { // HACK to prevent crash
+		OnChangeDiskAContents(event);
+		if (m_controller->IsOpenMSXRunning()) {
+			m_controller->WriteCommand(wxT("diska eject"));
+			if (!m_diskA->contents.IsEmpty()) {
+				m_controller->WriteCommand(wxT("diska ") + ConvertPath(m_diskA->contents,true));
+			}
 		}
 	}
 }
@@ -500,12 +506,16 @@ void SessionPage::OnChangeDiskAContents(wxCommandEvent & event)
 
 void SessionPage::OnClickDiskBCombo(wxCommandEvent & event)
 {
+	wxComboBox * box = (wxComboBox *)event.GetEventObject();
+	wxString sel = box->GetString(box->GetSelection());
 	OnClickCombo(event);
-	OnChangeDiskBContents(event);
-	if (m_controller->IsOpenMSXRunning()) {
-		m_controller->WriteCommand(wxT("diskb eject"));
-		if (!m_diskB->contents.IsEmpty()) {
-			m_controller->WriteCommand(wxT("diskb ") + ConvertPath(m_diskB->contents,true));
+	if (sel != box->GetString(box->GetSelection())) { // HACK to prevent crash
+		OnChangeDiskBContents(event);
+		if (m_controller->IsOpenMSXRunning()) {
+			m_controller->WriteCommand(wxT("diskb eject"));
+			if (!m_diskB->contents.IsEmpty()) {
+				m_controller->WriteCommand(wxT("diskb ") + ConvertPath(m_diskB->contents,true));
+			}
 		}
 	}
 }
@@ -529,12 +539,15 @@ void SessionPage::UpdateMenuMapperLabel(mediaInfo * target)
 void SessionPage::OnClickCartACombo(wxCommandEvent & event)
 {
 	OnClickCombo(event);
-	m_cartA->ips.Clear();
-	m_cartA->contents = m_cartA->control->GetValue();
-	m_cartA->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
-	m_cartA->type = m_cartA->typehistory[event.GetInt()];
-	UpdateMenuMapperLabel(m_cartA);
-	m_cartA->avoid_evt = true;
+	if (m_cartA->control->GetValue() != m_cartA->oldContents) { // HACK to prevent crash
+		m_cartA->ips.Clear();
+		m_cartA->contents = m_cartA->control->GetValue();
+		m_cartA->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
+		m_cartA->type = m_cartA->typehistory[event.GetInt()];
+		UpdateMenuMapperLabel(m_cartA);
+		m_cartA->avoid_evt = true;
+		m_cartA->oldContents = m_cartA->contents;
+	}
 }
 
 void SessionPage::OnChangeCartAContents(wxCommandEvent & event)
@@ -545,6 +558,7 @@ void SessionPage::OnChangeCartAContents(wxCommandEvent & event)
 		m_cartA->contents = m_cartA->control->GetValue();
 		m_cartA->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
 		UpdateMenuMapperLabel(m_cartA);
+		m_cartA->oldContents = m_cartA->contents;
 	}
 	m_cartA->avoid_evt = false;
 }
@@ -552,12 +566,15 @@ void SessionPage::OnChangeCartAContents(wxCommandEvent & event)
 void SessionPage::OnClickCartBCombo(wxCommandEvent & event)
 {
 	OnClickCombo(event);
-	m_cartB->ips.Clear();
-	m_cartB->contents = m_cartB->control->GetValue();
-	m_cartB->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
-	m_cartB->type = m_cartB->typehistory[event.GetInt()];
-	UpdateMenuMapperLabel(m_cartB);
-	m_cartB->avoid_evt = true;
+	if (m_cartB->control->GetValue() != m_cartB->oldContents) { // HACK to prevent crash
+		m_cartB->ips.Clear();
+		m_cartB->contents = m_cartB->control->GetValue();
+		m_cartB->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
+		m_cartB->type = m_cartB->typehistory[event.GetInt()];
+		UpdateMenuMapperLabel(m_cartB);
+		m_cartB->avoid_evt = true;
+		m_cartB->oldContents = m_cartB->contents;
+	}
 }
 
 void SessionPage::OnChangeCartBContents(wxCommandEvent & event)
@@ -567,7 +584,8 @@ void SessionPage::OnChangeCartBContents(wxCommandEvent & event)
 		m_cartB->type=wxT("");
 		m_cartB->contents = m_cartB->control->GetValue();
 		m_cartB->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
-		UpdateMenuMapperLabel(m_cartA);
+		UpdateMenuMapperLabel(m_cartB);
+		m_cartB->oldContents = m_cartB->contents;
 	}
 	m_cartB->avoid_evt = false;
 }
@@ -1070,6 +1088,7 @@ void SessionPage::RestoreHistory()
 		if ((m_InsertedMedia & (1 << i)) && (media[i]->history.GetCount() >0)) {
 			media[i]->control->SetSelection(0);
 			media[i]->contents=media[i]->history[0];
+			media[i]->oldContents=media[i]->history[0];
 			if ((media[i]==m_cartA) || (media[i]==m_cartB)){
 				media[i]->type=media[i]->typehistory[0];
 				UpdateMenuMapperLabel(media[i]);
@@ -1078,6 +1097,7 @@ void SessionPage::RestoreHistory()
 		else {
 			media[i]->control->SetValue(wxT(""));
 			media[i]->contents=wxT("");
+			media[i]->oldContents=wxT("");
 		}
 	}
 	config->GetParameter(ConfigurationData::CD_USEDMACHINE, m_usedMachine);
@@ -1393,59 +1413,48 @@ void SessionPage::OnBrowseDiskIps(wxCommandEvent & event)
 void SessionPage::GetRomTypes ()
 {
 	// TODO get it from openMSX
+	// NOTE: this list is incomplete! REALLY GET IT FROM OPENMSX!
 	SetupRomType(wxT(""),wxT("Autodetect type"));
 	SetupRomType(wxT("AUTO"),wxT("Autodetect type"));
-	SetupRomType(wxT("PAGE0"),wxT("Plain 16 kB Page 0"));
-	SetupRomType(wxT("PAGE1"),wxT("Plain 16 kB Page 1"));
-	SetupRomType(wxT("PAGE2"),wxT("Plain 16 kB Page 2 (Basic)"));
-	SetupRomType(wxT("ROMBAS"),wxT("Plain 16 kB Page 2 (Basic)"));
-	SetupRomType(wxT("PAGE3"),wxT("Plain 16 kB Page 3"));
-	SetupRomType(wxT("PAGE01"),wxT("Plain 32 kB Page 0-1"));
-	SetupRomType(wxT("PAGE12"),wxT("Plain 32 kB Page 1-2"));
-	SetupRomType(wxT("PAGE23"),wxT("Plain 32 kB Page 2-3"));
-	SetupRomType(wxT("PAGE012"),wxT("Plain 48 kB Page 0-2"));
-	SetupRomType(wxT("PAGE123"),wxT("Plain 48 kB Page 1-3"));
-	SetupRomType(wxT("PAGE0123"),wxT("Plain 64 kB"));
-	SetupRomType(wxT("PLAIN"),wxT("Plain (Any size)"));
-	SetupRomType(wxT("64kB"),wxT("Plain (Any size)"));
-//	SetupRomType(wxT("DRAM"),wxT("DRAM"));
+	SetupRomType(wxT("Page0"),wxT("Plain 16 kB Page 0"));
+	SetupRomType(wxT("Page1"),wxT("Plain 16 kB Page 1"));
+	SetupRomType(wxT("Page2"),wxT("Plain 16 kB Page 2 (Basic)"));
+	SetupRomType(wxT("Normal8000"),wxT("Plain 16 kB Page 2 (Basic)"));
+	SetupRomType(wxT("Page3"),wxT("Plain 16 kB Page 3"));
+	SetupRomType(wxT("Page01"),wxT("Plain 32 kB Page 0-1"));
+	SetupRomType(wxT("Page12"),wxT("Plain 32 kB Page 1-2"));
+	SetupRomType(wxT("Page23"),wxT("Plain 32 kB Page 2-3"));
+	SetupRomType(wxT("Page012"),wxT("Plain 48 kB Page 0-2"));
+	SetupRomType(wxT("Page123"),wxT("Plain 48 kB Page 1-3"));
+	SetupRomType(wxT("Page0123"),wxT("Plain 64 kB"));
+	SetupRomType(wxT("Mirrored"),wxT("Plain (Any size)"));
 	SetupRomType(wxT("8kB"),wxT("Generic 8kB"));
-	SetupRomType(wxT("0"),wxT("Generic 8kB"));
 	SetupRomType(wxT("16kB"),wxT("Generic 16kB"));
-	SetupRomType(wxT("1"),wxT("Generic 16kB"));
-	SetupRomType(wxT("MSXDOS2"),wxT("Generic 16kB"));
-	SetupRomType(wxT("KONAMI4"),wxT("Konami without SCC"));
-	SetupRomType(wxT("3"),wxT("Konami without SCC"));
-	SetupRomType(wxT("KONAMI5"),wxT("Konami with SCC"));
-	SetupRomType(wxT("SCC"),wxT("Konami with SCC"));
-	SetupRomType(wxT("2"),wxT("Konami with SCC"));
+	SetupRomType(wxT("Konami"),wxT("Konami without SCC"));
+	SetupRomType(wxT("KonamiSCC"),wxT("Konami with SCC"));
 	SetupRomType(wxT("ASCII8"),wxT("Ascii 8kB"));
-	SetupRomType(wxT("4"),wxT("Ascii 8kB"));
 	SetupRomType(wxT("ASCII16"),wxT("Ascii 16kB"));
-	SetupRomType(wxT("5"),wxT("Ascii 16kB"));
-	SetupRomType(wxT("RTYPE"),wxT("R-Type"));
-	SetupRomType(wxT("CROSSBLAIM"),wxT("Cross Blaim"));
-//	SetupRomType(wxT("PANASONIC"),wxT("Panasonic"));
-//	SetupRomType(wxT("NATIONAL"),wxT("National"));
+	SetupRomType(wxT("Padial8"),wxT("Padial 8kB"));
+	SetupRomType(wxT("Padial16"),wxT("Padial 16kB"));
+	SetupRomType(wxT("R-Type"),wxT("R-Type"));
+	SetupRomType(wxT("CrossBlaim"),wxT("Cross Blaim"));
 	SetupRomType(wxT("MSX-AUDIO"),wxT("MSX-Audio"));
-	SetupRomType(wxT("HARRYFOX"),wxT("Harry Fox"));
-	SetupRomType(wxT("HALNOTE"),wxT("Halnote"));
-	SetupRomType(wxT("KOREAN80IN1"),wxT("Korean Multigame (80 in 1)"));
-	SetupRomType(wxT("KOREAN90IN1"),wxT("Korean Multigame (90 in 1)"));
-	SetupRomType(wxT("KOREAN126IN1"),wxT("Korean Multigame (126 in 1)"));
-	SetupRomType(wxT("HOLYQURAN"),wxT("Holy Qu'ran"));
-//	SetupRomType(wxT("FSA1FM1"),wxT("Panasonic FS-A1FM Rom1"));
-//	SetupRomType(wxT("FSA1FM2"),wxT("Panasonic FS-A1FM Rom2"));
-	SetupRomType(wxT("ASCII16-2"),wxT("Ascii 16kB with SRAM"));
-	SetupRomType(wxT("HYDLIDE2"),wxT("Ascii 16kB with SRAM"));
-	SetupRomType(wxT("ASCII8-8"),wxT("Ascii 8kB with SRAM"));
-	SetupRomType(wxT("KOEI-8"),wxT("Koei with 8kB SRAM"));
-	SetupRomType(wxT("KOEI-32"),wxT("Koei with 32kB SRAM"));
+	SetupRomType(wxT("HarryFox"),wxT("Harry Fox"));
+	SetupRomType(wxT("Halnote"),wxT("Halnote"));
+	SetupRomType(wxT("Playball"),wxT("Playball"));
+	SetupRomType(wxT("Zemina80in1"),wxT("Zemina Multigame (80 in 1)"));
+	SetupRomType(wxT("Zemina90in1"),wxT("Zemina Multigame (90 in 1)"));
+	SetupRomType(wxT("Zemina126in1"),wxT("Zemina Multigame (126 in 1)"));
+	SetupRomType(wxT("HolyQuran"),wxT("Holy Qu'ran"));
+	SetupRomType(wxT("ASCII16SRAM2"),wxT("Ascii 16kB with SRAM"));
+	SetupRomType(wxT("ASCII8SRAM8"),wxT("Ascii 8kB with SRAM"));
+	SetupRomType(wxT("KoeiSRAM8"),wxT("Koei with 8kB SRAM"));
+	SetupRomType(wxT("KoeiSRAM32"),wxT("Koei with 32kB SRAM"));
 	SetupRomType(wxT("Wizardry"),wxT("Wizardry"));
-	SetupRomType(wxT("GAMEMASTER2"),wxT("Konami Game Master 2"));
+	SetupRomType(wxT("GameMaster2"),wxT("Konami Game Master 2"));
+	SetupRomType(wxT("Synthesizer"),wxT("Konami Synthesizer"));
 	SetupRomType(wxT("RC755"),wxT("Konami Game Master 2"));
-	SetupRomType(wxT("SYNTHESIZER"),wxT("Konami Synthesizer"));
-	SetupRomType(wxT("MAJUTSUSHI"),wxT("Hai no Majutsushi"));
+	SetupRomType(wxT("Majutsushi"),wxT("Hai no Majutsushi"));
 }
 
 void SessionPage::AddRomType(wxString type)
