@@ -1,4 +1,4 @@
-// $Id: openMSXController.cpp,v 1.99 2006/07/08 12:45:15 mthuurne Exp $
+// $Id: openMSXController.cpp,v 1.105 2007/03/01 20:25:45 manuelbi Exp $
 // openMSXController.cpp: implementation of the openMSXController class.
 //
 //////////////////////////////////////////////////////////////////////
@@ -249,7 +249,7 @@ void openMSXController::HandleParsedOutput(wxCommandEvent &event)
 					}
 					else{
 						wxString command = GetPendingCommand();
-						if (command == GetInfoCommand(wxT("fps"))) {
+						if (command == wxT("openmsx_info fps")) {
 							m_appWindow->SetFPSdisplay(data->contents);
 						}
 						else if (command == wxT("save_settings")){
@@ -418,10 +418,6 @@ void openMSXController::HandleNormalLaunchReply(wxCommandEvent &event)
 	executeLaunch(&event);
 }
 
-wxString openMSXController::GetInfoCommand (wxString parameter)
-{
-	return wxString (wxT("join [openmsx_info ") + parameter +wxT("] \\n"));
-}
 
 int openMSXController::InitConnectors(wxString dummy, wxString connectors)
 {
@@ -566,21 +562,14 @@ bool openMSXController::SetupOpenMSXParameters(wxString version)
 	}
 	// printf ("Detected openMSX version: %d\n", ver);
 	if (ver == -1) {
-		wxMessageBox (wxT("Unable to determine openMSX version!\nPlease upgrade to 0.6.0 or higher.\n(Or contact the authors.)"),wxT("Error"));
+		wxMessageBox (wxT("Unable to determine openMSX version!\nPlease upgrade to 0.6.2 or higher.\n(Or contact the authors.)"),wxT("Error"));
 		return false;
 	}
-	if (ver < 600) {
-		wxMessageBox (wxT("The openMSX version you are using is too old!\nPlease upgrade to 0.6.0 or higher."),wxT("Error"));
+	if (ver < 602) {
+		wxMessageBox (wxT("The openMSX version you are using is too old!\nPlease upgrade to 0.6.2 or higher."),wxT("Error"));
 		return false;
 	}
-	if ((version.Find(wxT("-dev")) != -1) || (ver > 502)) {
-		// put specific code for NEW versions here
-		m_appWindow->m_miscControlPage->FillInitialJoystickPortValues(2);
-	}
-	else{
-		// put specific code for OLD supported versions here
-		m_appWindow->m_miscControlPage->FillInitialJoystickPortValues(1);
-	}
+	m_appWindow->m_miscControlPage->FillInitialJoystickPortValues();
 	m_appWindow->m_launch_AbortButton->Enable(true);
 	return true;
 }
@@ -594,10 +583,10 @@ void openMSXController::InitLaunchScript ()
 	AddLaunchInstruction (wxT("set power on"),wxT("e"),wxT("power"),&openMSXController::UpdateSetting,true);
 	AddLaunchInstruction (wxT("unset renderer"),wxT("e"),wxT(""),NULL,true);
 	AddLaunchInstruction (wxT("@execute"),wxT(""),wxT(""),&openMSXController::EnableMainWindow,false);
-	AddLaunchInstruction (wxT("#info renderer"),wxT(""),wxT("RendererSelector"),&openMSXController::FillComboBox,true);
-	AddLaunchInstruction (wxT("#info scale_algorithm"),wxT(""),wxT("ScalerAlgoSelector"),&openMSXController::FillComboBox,true);
+	AddLaunchInstruction (wxT("^info renderer"),wxT(""),wxT("RendererSelector"),&openMSXController::FillComboBox,true);
+	AddLaunchInstruction (wxT("^info scale_algorithm"),wxT(""),wxT("ScalerAlgoSelector"),&openMSXController::FillComboBox,true);
 	AddLaunchInstruction (wxT("lindex [openmsx_info setting scale_factor] 2"),wxT(""),wxT("ScalerFactorSelector"),&openMSXController::FillRangeComboBox,true);
-	AddLaunchInstruction (wxT("#info accuracy"),wxT(""),wxT("AccuracySelector"),&openMSXController::FillComboBox,false);
+	AddLaunchInstruction (wxT("^info accuracy"),wxT(""),wxT("AccuracySelector"),&openMSXController::FillComboBox,false);
 	AddLaunchInstruction (wxT("update enable media"),wxT(""),wxT(""),NULL,false);
 	AddLaunchInstruction (wxT("info exist frontswitch"),wxT(""),wxT("#"),&openMSXController::EnableFirmware,false);
 	AddLaunchInstruction (wxT("info exist firmwareswitch"),wxT(""),wxT("#"),&openMSXController::EnableFirmware,false);
@@ -622,11 +611,11 @@ void openMSXController::InitLaunchScript ()
 	AddLaunchInstruction (wxT("set minframeskip"),wxT(""),wxT("minframeskip"),&openMSXController::UpdateSetting,true);
 	AddLaunchInstruction (wxT("set throttle"),wxT(""),wxT("throttle"),&openMSXController::UpdateSetting,true);
 	AddLaunchInstruction (wxT("set cmdtiming"),wxT(""),wxT("cmdtiming"),&openMSXController::UpdateSetting,true);
-	AddLaunchInstruction (wxT("#info pluggable"),wxT("13"),wxT(""),&openMSXController::InitPluggables,true);
-	AddLaunchInstruction (wxT("#info_nostore pluggable *"),wxT(""),wxT("*"),&openMSXController::AddPluggableDescription,true);
-	AddLaunchInstruction (wxT("#info_nostore connectionclass *"),wxT(""),wxT("*"),&openMSXController::AddPluggableClass,false);
-	AddLaunchInstruction (wxT("#info connector"),wxT("10"),wxT(""),&openMSXController::InitConnectors,true);
-	AddLaunchInstruction (wxT("#info_nostore connectionclass *"),wxT(""),wxT("*"),&openMSXController::AddConnectorClass,false);
+	AddLaunchInstruction (wxT("!info pluggable"),wxT("13"),wxT(""),&openMSXController::InitPluggables,true);
+	AddLaunchInstruction (wxT("!info_nostore pluggable *"),wxT(""),wxT("*"),&openMSXController::AddPluggableDescription,true);
+	AddLaunchInstruction (wxT("!info_nostore connectionclass *"),wxT(""),wxT("*"),&openMSXController::AddPluggableClass,false);
+	AddLaunchInstruction (wxT("!info connector"),wxT("10"),wxT(""),&openMSXController::InitConnectors,true);
+	AddLaunchInstruction (wxT("!info_nostore connectionclass *"),wxT(""),wxT("*"),&openMSXController::AddConnectorClass,false);
 	AddLaunchInstruction (wxT("plug *"),wxT(""),wxT("*"),&openMSXController::AddConnectorContents,true);
 	AddLaunchInstruction (wxT("@checkfor msx-midi-in"),wxT("1"),wxT(""),NULL,false);
 	AddLaunchInstruction (wxT("set midi-in-readfilename"),wxT(""),wxT("midi-in-readfilename"),&openMSXController::UpdateSetting,true);
@@ -638,8 +627,8 @@ void openMSXController::InitLaunchScript ()
 	AddLaunchInstruction (wxT("@execute"),wxT(""),wxT(""),&openMSXController::InitAudioConnectorPanel,false);
 //	AddLaunchInstruction (wxT("#info romtype"),wxT(""),wxT(""),&openMSXController::InitRomTypes,true);
 //	AddLaunchInstruction (wxT("#info_nostore romtype *"),wxT(""),wxT("*"),&openMSXController::SetRomDescription,true);
-	AddLaunchInstruction (wxT("#info sounddevice"),wxT("5"),wxT(""),&openMSXController::InitSoundDevices,true);
-	AddLaunchInstruction (wxT("#info_nostore sounddevice *"),wxT(""),wxT("*"),&openMSXController::SetChannelType,true);
+	AddLaunchInstruction (wxT("!info sounddevice"),wxT("5"),wxT(""),&openMSXController::InitSoundDevices,true);
+	AddLaunchInstruction (wxT("!info_nostore sounddevice *"),wxT(""),wxT("*"),&openMSXController::SetChannelType,true);
 	AddLaunchInstruction (wxT("set master_volume"),wxT(""),wxT("master_volume"),&openMSXController::UpdateSetting,false);
 	AddLaunchInstruction (wxT("set *_volume"),wxT(""),wxT("*_volume"),&openMSXController::UpdateSetting,true);
 	AddLaunchInstruction (wxT("set *_mode"),wxT(""),wxT("*_mode"),&openMSXController::UpdateSetting,true);
@@ -703,6 +692,12 @@ void openMSXController::executeLaunch (wxCommandEvent * event, int startLine)
 			if (tokens[0] == wxT("#info")) {
 				lastdata = data->contents;
 			}
+			if (tokens[0] == wxT("^info")) {
+				lastdata = data->contents;
+			}
+			if (tokens[0] == wxT("!info")) {
+				lastdata = data->contents;
+			}
 			HandleLaunchReply (cmd,event,m_launchScript[recvStep],&sendStep,recvLoop,lastdata);
 			if ((data->replyState == CatapultXMLParser::REPLY_NOK) ||
 				((cmd.Mid(0,10) == wxT("info exist")) && (data->contents == wxT("0")))) {
@@ -877,7 +872,37 @@ wxString openMSXController::translate(wxArrayString tokens, int loop, wxString l
 					tokens.RemoveAt(token+1);
 				}
 				parameter.Trim(true);
-				tokens[token] = GetInfoCommand(parameter);
+				tokens[token] = wxString(wxT("join [openmsx_info ") + parameter + wxT("] \\n"));
+			}
+			else {
+				assert(false); // invalid command
+			}
+			break;
+		case '!':
+			if (tokens[token].Mid(0,5)== wxT("!info")) {
+				wxString parameter = wxT("");
+				while (token < (tokens.GetCount()-1)) {
+					parameter += tokens[token+1];
+					parameter += wxT(" ");
+					tokens.RemoveAt(token+1);
+				}
+				parameter.Trim(true);
+				tokens[token] = wxString(wxT("join [machine_info ") + parameter + wxT("] \\n"));
+			}
+			else {
+				assert(false); // invalid command
+			}
+			break;
+		case '^':
+			if (tokens[token].Mid(0,5)== wxT("^info")) {
+				wxString parameter = wxT("");
+				while (token < (tokens.GetCount()-1)) {
+					parameter += tokens[token+1];
+					parameter += wxT(" ");
+					tokens.RemoveAt(token+1);
+				}
+				parameter.Trim(true);
+				tokens[token] = wxString(wxT("join [lindex [openmsx_info setting ") + parameter + wxT("] 2] \\n"));
 			}
 			else {
 				assert(false); // invalid command
